@@ -44,6 +44,8 @@ POSTGRES_INT_MIN = -2147483648
 POSTGRES_BIGINT_MAX = 9223372036854775807
 POSTGRES_BIGINT_MIN = -9223372036854775808
 
+MINIMUM_QSV_VERSION = "0.87.1"
+
 DATASTORE_URLS = {
     "datastore_delete": "{ckan_url}/api/action/datastore_delete",
     "resource_update": "{ckan_url}/api/action/resource_update",
@@ -389,10 +391,10 @@ def push_to_datastore(task_id, input, dry_run=False):
         qsv_version_info.find(" ") : qsv_version_info.find("-")
     ].lstrip()
     try:
-        if semver.compare(qsv_semver, "0.85.0") < 0:
+        if semver.compare(qsv_semver, MINIMUM_QSV_VERSION) < 0:
             raise util.JobError(
-                "At least qsv version 0.85.0 required. Found {}. You can get the latest release at https://github.com/jqnatividad/qsv/releases/latest".format(
-                    qsv_version_info
+                "At least qsv version {} required. Found {}. You can get the latest release at https://github.com/jqnatividad/qsv/releases/latest".format(
+                    MINIMUM_QSV_VERSION, qsv_version_info
                 )
             )
     except ValueError as e:
@@ -1633,11 +1635,16 @@ def push_to_datastore(task_id, input, dry_run=False):
     # If AUTO_INDEX_DATES is true, index all date columns
     # if a column's cardinality <= AUTO_INDEX_THRESHOLD, create an index for that column
     auto_index_dates = config.get("AUTO_INDEX_DATES")
-    if auto_index_threshold or (auto_index_dates and datetimecols_list):
+    auto_unique_index = config.get("AUTO_UNIQUE_INDEX")
+    if (
+        auto_index_threshold
+        or (auto_index_dates and datetimecols_list)
+        or auto_unique_index
+    ):
         index_start = time.perf_counter()
         logger.info(
-            "AUTO-INDEXING. Auto-index threshold: {:,} unique value/s. Auto-index dates: {} ...".format(
-                auto_index_threshold, auto_index_dates
+            "AUTO-INDEXING. Auto-index threshold: {:,} unique value/s. Auto-unique index: {} Auto-index dates: {} ...".format(
+                auto_index_threshold, auto_unique_index, auto_index_dates
             )
         )
         index_cur = raw_connection.cursor()
@@ -1651,10 +1658,13 @@ def push_to_datastore(task_id, input, dry_run=False):
         for idx, cardinality in enumerate(headers_cardinality):
 
             curr_col = headers[idx]
-            if auto_index_threshold > 0 or auto_index_dates:
-                if cardinality == record_count:
+            if auto_index_threshold > 0 or auto_index_dates or auto_unique_index:
+                if cardinality == record_count and auto_unique_index:
                     # all the values are unique for this column, create a unique index
-                    unique_value_count = min(preview_rows, cardinality)
+                    if preview_rows > 0:
+                        unique_value_count = min(preview_rows, cardinality)
+                    else:
+                        unique_value_count = cardinality
                     logger.info(
                         'Creating UNIQUE index on "{}" for {:,} unique values...'.format(
                             curr_col, unique_value_count
